"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const runner_1 = require("./runner");
const Gherkin = require('gherkin');
const toposort = require('toposort');
const globAsync = require("glob");
const util_1 = require("util");
const path = require("path");
const glob = util_1.promisify(globAsync);
const fs_1 = require("fs");
const parser = new Gherkin.Parser(new Gherkin.AstBuilder());
const matcher = new Gherkin.TokenMatcher();
exports.parseFeatures = (featureData) => {
    const parsedFeatures = featureData.map(d => {
        // Parse the feature files
        const scanner = new Gherkin.TokenScanner(d.toString());
        return parser.parse(scanner, matcher).feature;
    });
    const featureNames = parsedFeatures.map(({ name }) => name);
    // Sort the features by the step 'I am run after the "..." feature' using toposort
    const featureDependencies = parsedFeatures.map(feature => {
        const bg = feature.children.find(({ type }) => type === 'Background');
        if (bg) {
            const afterStep = bg.steps.find(({ text }) => runner_1.afterRx.test(text));
            if (afterStep) {
                const m = afterStep.text.match(runner_1.afterRx);
                if (!m) {
                    throw new Error(`Failed to find feature in ${afterStep.text}`);
                }
                if (!featureNames.includes(m[1])) {
                    throw new Error(`The feature ${m[1]} you want to run after does not exist!`);
                }
                return [m[1], feature.name];
            }
        }
        return [feature.name, false];
    });
    const sortedFeatureNames = toposort(featureDependencies).filter((feature) => feature);
    // Now bring the features in the right order
    const sortedFeatures = sortedFeatureNames.map((featureName) => parsedFeatures.find(({ name }) => name === featureName));
    // Find features to be skipped
    const only = parsedFeatures
        .filter(({ tags }) => tags.find(({ name }) => name === '@Only'))
        .map(({ name }) => name);
    return sortedFeatures.map(f => {
        const { tags, name: featureName } = f;
        const skip = tags.find(({ name }) => name === '@Skip') ||
            (only.length && !only.includes(featureName));
        return Object.assign({}, f, { skip: !!skip });
    });
};
exports.fromDirectory = async (dir) => {
    const scan = path.join(path.resolve(dir), '*.feature');
    const featureFiles = await glob(scan);
    return exports.parseFeatures(featureFiles.map(f => fs_1.readFileSync(f)));
};
